------------------------------------------------------State functions

function pushState(state)
	table.insert(states,state)
	state:setCallbacks()
end

function switchState(state)	--switches with the state on top of the stack
	popState()
	pushState(state)
end

function popState()
	states[#states]:destroy()
	table.remove(states,#states)
	
	if next(states) then
		states[#states]:setCallbacks()
	end
end

-------------------------------------------------------State Classes
--[[
There are two ways to implement the actual classes:
1.) You can have a superclass that has the needed functions and members defined and then make subclasses that inherit/override those members and functions
2.) You can make StateClasses and just make sure they always have the appropriate members/methods.

Regardless, if you create an instance of a class, and intend to use it with the state functions and global states table, make sure it has members:
alwaysUpdate = true/false
alwaysDraw = true/false

and make sure it has methods:

update
draw
setCallbacks
destroy

I prefer the second method, so my state classes might look like:
]]

TitleState = class('TitleState')

function TitleState:initialize()
	self.alwaysUpdate = false
	self.alwaysDraw = true
	self.font20 = love.graphics.newFont(20)
	self.font12 = love.graphics.newFont(12)
end

function TitleState:update(dt)

end

function TitleState:draw()
	local lg = love.graphics
	lg.setColor(20,20,40)
	lg.rectangle('fill',10,10,sw-20,sh-20)
	lg.setColor(18,22,144)
	lg.setFont(self.font20)
	lg.printf('Super Block Move 4KX000',300,200,sw-600,'center')
	lg.setColor(255,255,255)
	lg.printf('press x to begin', 20,sh-40,300,'left')
	lg.setFont(self.font12)
end

function TitleState:destroy()

end

function TitleState:setCallbacks()
	function love.keypressed(k,uc)
		if k == 'x' then
			--note, the playstate is created as a global
			play = PlayState()
			switchState(play)
		end
	end
end

-----------------------------------

PlayState = class('PlayState')

function PlayState:initialize()
	self.alwaysUpdate = false
	self.alwaysDraw = true
	self.player = {x = 300,y = 400}
	self.playerColor = {255,255,255}
end

function PlayState:update(dt)
	self.playerColor[1] = self.playerColor[1] + 13
	self.playerColor[2] = self.playerColor[2] + 6
	self.playerColor[3] = self.playerColor[3] + 2
	if self.player.x > sw-25 then self.player.x = sw-25 end
	if self.player.x < 0 then self.player.x = 0 end
	if self.player.y > sh-25 then self.player.y = sh-25 end
	if self.player.y < 0 then self.player.y = 0 end
end

function PlayState:draw()
	local lg = love.graphics
	lg.setColor(self.playerColor)
	lg.rectangle('fill',self.player.x,self.player.y,25,25)
	lg.setColor(255,255,255)
	lg.print('press q to quit playing', 10,10)
end

function PlayState:destroy()

end

function PlayState:setCallbacks()
	function love.keypressed(k,uc)
		if k == 'a' then
			self.player.x = self.player.x - 20
		elseif k == 'd' then
			self.player.x = self.player.x + 20
		elseif k== 'w' then
			self.player.y = self.player.y - 20
		elseif k == 's' then
			self.player.y = self.player.y + 20
		elseif k == 'q' then
			prompt = PromptState()
			pushState(prompt)
		else
		
		end
	end
end

---------------------------------------

PromptState = class('PromptState')

function PromptState:initialize()
	self.alwaysUpdate = false
	self.alwaysDraw = true
end

function PromptState:update(dt)

end

function PromptState:draw()
	local lg = love.graphics
	lg.setColor(200,200,200,95)
	lg.rectangle('fill',0,0,sw,sh)
	lg.setColor(220,80,20,127)
	lg.rectangle('line',300,250,200,100)
	lg.setColor(0,0,0)
	lg.printf([[If you're sure, press 'q' again]], 300,250,200,'center')
end

function PromptState:destroy()

end

function PromptState:setCallbacks()
	function love.keypressed(k,uc)
		if k == 'q' then
			popState()
			switchState(title)		--switches play with title
		end
	end
end
